package jlegocom;

import java.io.*;
import java.net.*;
import java.util.*;


/****************************************************************
****************************************************************/
public class LegoAdapter extends Object
{
	public ServerSocket toJavaServer, toRobotServer;
	private Ping ping;
	private Timer timer;
	private WorkTimer workTimer;
	
	static final int		DELEGATE_PORT = 6133,
											TO_ROBOT_SERVER_PORT = 6134;
	
	RobotCommThread robotThread;		//thread for robot-communication
	Vector commThreads;  						//threads for java-clients	
	DelegateEventThread theChiefApplet;
	int num = 0;
	boolean robotAlive = false;
	
	Vector allCmds = new Vector();
	
	static final boolean WORK = false;

	
	/*************************************************************************
  *************************************************************************/
	public LegoAdapter()
	{
		commThreads=new Vector();
		System.out.println("-------------------------------------------------");
		System.out.println(" LegoAdapter (Java-Server) Version 0.1");
		System.out.println("-------------------------------------------------");

		// first open server socket in order to communicate with robot
		//this.openSocketToRobot();
		RobotAcceptingThread acceptingThread = new RobotAcceptingThread(this);
		acceptingThread.start();
		
		// open server socket for communication with java clients
		try{
			toJavaServer = new ServerSocket(DELEGATE_PORT);
		}
		catch(IOException ex){
			ex.printStackTrace();
		}
		
		//start ping-thread
		ping = new Ping(this);
		ping.start();
		
		//start timer 5min
		if (WORK) {
			workTimer = new WorkTimer(this);
			workTimer.start();
		}
		
		// accepting loop for java clients
		startAccepting();
	}

	/*************************************************************************
  *************************************************************************/
	public static void main(String[] args)
	{
		new LegoAdapter();
	}

	/*************************************************************************
   *Watches out for new JavaClients.
  *************************************************************************/	
	private void startAccepting()
	{
		System.out.println("O Start accepting applets...");
		while(true){
			try{
				createCommThread(toJavaServer.accept());
				
				// checks if one thread(client) died 
				checkAllThreads();
				System.out.println("O LegoAdapter accepted one new Applet! now:"+commThreads.size());
			}
			catch(IOException e)
			{
				System.out.println("X Problem while connecting applet: "+e);
			}
		}
	}

	/*************************************************************************
   *Opens a serversocket for communication with the robot.
  *************************************************************************/
	public void openSocketToRobot()
	{
		System.out.println("..open new serversocket to ROBOT..");
	
		try{
			toRobotServer = new ServerSocket(TO_ROBOT_SERVER_PORT);
		}
		catch(IOException ex){
			ex.printStackTrace();
		}
	}
	
	/*************************************************************************
   *Creates a Thread for communication with the robot.
  *************************************************************************/
	public void createRobotThread(Socket socket)
	{
		robotThread = new RobotCommThread("LegoAdapter", socket, this);
		robotThread.start();
		robotAlive = true;
		System.out.println("O Successfully opened ServerSocket to ROBOT...");
	}
	
	/*************************************************************************
   *Creates a new DelegateEventThread.
  *************************************************************************/	
	private void createCommThread(Socket s)
	{
		DelegateEventThread newT = new DelegateEventThread("DelegateThread"+num, s, this);
		newT.start();
		commThreads.addElement(newT);
		num++;
	}

	/*************************************************************************
   *Checks if a thread died and removes all dead threads.
  *************************************************************************/
	public void checkAllThreads()
	{
		for(int i=0; i < commThreads.size(); i++){
			DelegateEventThread t = (DelegateEventThread)commThreads.elementAt(i);
			if(!t.isAlive()){
				if (commThreads.size() > i){
					//commT.destroy();
					this.removeThread(i);
					i--;
					System.out.println(t.getName()+" not alive! -> removed!");
				}
			}
			else
				System.out.println(t.getName()+" alive: "+t.isAlive());
		}
	}

	/*************************************************************************
   *Stop dead thread.
  *************************************************************************/
	public void stopMe(Thread t)
	{
		//entsprechenden thread aus commThreads suchen und entfernen!
		DelegateEventThread delT = null;
		if (commThreads.size() > 0) {
			for (int i = 0; i < commThreads.size(); i++){
				delT = (DelegateEventThread)commThreads.elementAt(i);
				if(delT.getName().equals(t.getName())){
					if (commThreads.size() > i){
						System.out.println(t.getName()+" terminated!");
						//delT.destroy();
						this.removeThread(i);
						break;
					}
				}
			}
		}
	}

	/*************************************************************************
   *Stop robot thread.
  *************************************************************************/
	public void stopRobotThread()
	{
		if (robotThread != null){
			System.out.println(robotThread.getName()+" terminated!");
			robotThread = null;
			robotAlive = false;
		}
	}
	
	/*************************************************************************
   *sends command to all JavaAdapters.
  *************************************************************************/
	public void sendToAdapters(String cmd)
	{
		DelegateEventThread commT = null;
			
		if (commThreads.size() > 0) {
			for (int i = 0; i < commThreads.size(); i++){
				commT = (DelegateEventThread)commThreads.elementAt(i);
				String hostname = commT.getHostName();
				if(commT.isAlive()){
					if (cmd.startsWith("JLEGO")){
						try{
							commT.oOutStr.writeObject(cmd);
							System.out.println(commT.getName()+": SENT TO APPLET "+i+" on ["+commT.getHostName()+"] cmd: <"+cmd+">");
						}
						catch(IOException e) {
							System.out.println(commT.getName()+": Couldn't get I/O for connection! -> removed!");
							if (commThreads.size() > i){
								//commT.destroy();
								this.removeThread(i);
								i--;
							}
						}
					}
				}
				else{
					if (commThreads.size() > i){
						this.removeThread(i);
						System.out.println(commT.getName()+" not alive! -> removed!");
						i--;
					}
				}
			}
		}
		else
			System.out.println("No Applets available for sending Command:<"+cmd+">");
	}
	
	/*************************************************************************
   *Sends a command to one special JavaAdapters.
  *************************************************************************/
	public boolean sendToAdapterX(String cmdHost, String threadName, String cmd)
	{
		boolean found = false;	
		DelegateEventThread commT = null;

		if (commThreads.size() > 0) {
			for (int i = 0; i < commThreads.size(); i++){
				
				commT = (DelegateEventThread)commThreads.elementAt(i);
				String hostName = commT.getHostName();
				
				System.out.println(i+": CmdHost: "+cmdHost+": ThisHost: "+hostName);
				
				if (commT.isAlive()){
					if ((cmdHost.equals(hostName)) && (commT.getName().equals(threadName))){
						try{
							commT.oOutStr.writeObject(cmd);
							System.out.println(commT.getName()+": SENT TO APPLET "+i+" on ["+hostName+"] cmd: <"+cmd+">");
							found = true;
						}
						catch(IOException e) {
							System.out.println(commT.getName()+": Couldn't get I/O for connection: "+e);
						}
					}
				}
				else if (commThreads.size() > i){
					this.removeThread(i);
					i--;
				}
			}
		}
		else
			System.out.println("No Applet available for sending Command:<"+cmd.toString()+">");
		return(found);
	}
	
	/*************************************************************************
   *Sends a command to all but not X JavaAdapters.
  *************************************************************************/
	public boolean sendToAdapterNotX(String cmdHost, String threadName, String cmd)
	{
		boolean found = false;	
		DelegateEventThread commT = null;

		if (commThreads.size() > 0) {
			for (int i = 0; i < commThreads.size(); i++){
				
				commT = (DelegateEventThread)commThreads.elementAt(i);
				String hostName = commT.getHostName();
									
				if (commT.isAlive()){
					if (!((cmdHost.equals(hostName)) && (commT.getName().equals(threadName)))){
						try{
							commT.oOutStr.writeObject(cmd);
							System.out.println(commT.getName()+": SENT TO APPLET "+i+" on ["+hostName+"] cmd: <"+cmd+">");
							found = true;
						}
						catch(IOException e) {
							System.out.println(commT.getName()+": Couldn't get I/O for connection: "+e);
						}
					}
				}
				else if (commThreads.size() > i){
					this.removeThread(i);
					i--;
				}
			}
		}
		else
			System.out.println("No Applet available for sending Command:<"+cmd.toString()+">");
		return(found);
	}

	/*************************************************************************
  *************************************************************************/
	public boolean sendToRobot(String cmd)
	{
		if ((robotThread != null) && robotAlive()) {
			robotThread.outStream.println(cmd);
			System.out.println(robotThread.getName()+": SENT TO ROBOT: <"+cmd+">");
			return(true);
		}
		else{
			System.out.println("No ROBOT available for sending Command!");
			return(false);
		}
	}

	/*************************************************************************
  *************************************************************************/
	public boolean robotAlive()
	{
		return(robotAlive);
	}
	
	/*************************************************************************
  *************************************************************************/
	public boolean freeControl()
	{
		return(theChiefApplet == null);
	}
	
	/*************************************************************************
  *************************************************************************/
	public boolean getControl(DelegateEventThread applet)
	{
		if (theChiefApplet == null){			
			//stop timer 5 min
			if (WORK) {
				workTimer.stop();
				workTimer = null;
			}
			
			//kill old timer
			if (timer != null){
				timer.stopme();
				timer = null;			
			}
			
			//start timer 10min
			timer = new Timer(this);
			timer.start();
			
			theChiefApplet = applet;
			return(true);
		}
		else return(false);
	}

	/*************************************************************************
  *************************************************************************/
	public void forgetControl()
	{
		//stop timer 10min
		if (timer != null){
			timer.stopme();
			timer = null;
		
			//start timer 5 min
			if (WORK) {
				if (workTimer == null){
					workTimer = new WorkTimer(this);
					workTimer.start();
				}
			}

			theChiefApplet = null;
			this.sendToAdapters("JLEGO OFFLINE");
			this.sendToAdapters("JLEGO FREE");
		}
	}

	/*************************************************************************
  *************************************************************************/
	public void removeThread(int i)
	{
		System.out.println("***REMOVE: "+((DelegateEventThread)commThreads.elementAt(i)).getName());
		if ((theChiefApplet != null) && (((DelegateEventThread)commThreads.elementAt(i)).getName().equals(theChiefApplet.getName()))) this.forgetControl();					
		this.commThreads.removeElementAt(i);
		for (int j = 0; j<commThreads.size(); j++)
			System.out.println(((Thread)commThreads.elementAt(i)).getName());
	}
}
